package analysisOperations;

import ij.IJ;
import ij.ImagePlus;
import ij.measure.ResultsTable;
import operations.Operation;

public class ComputeDifferenceOperation extends Operation {
	/**
	 * 
	 */
	private static final long serialVersionUID = -7770582616755118107L;
	protected ImagePlus firstImage;
	protected ImagePlus secondImage;
	protected ResultsTable difference;

	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		parameterTypes = new Class[2];
		parameterTypes[0] = Class.forName("ij.ImagePlus");
		parameterTypes[1] = Class.forName("ij.ImagePlus");
		parameterNames = new String[2];
		parameterNames[0] = "FirstImage";
		parameterNames[1] = "SecondImage";
		resultTypes = new Class[1];
		resultTypes[0] = Class.forName("ij.measure.ResultsTable");
		resultNames = new String[1];
		resultNames[0] = "Difference";
	}

	public ResultsTable getDifference() {
		if (difference==null) {
			difference = new ResultsTable();
		}
		return difference;
	}

	public void setDifference(ResultsTable difference) {
		this.difference = difference;
	}

	public ImagePlus getFirstImage() {
		return firstImage;
	}

	public void setFirstImage(ImagePlus firstImage) {
		this.firstImage = firstImage;
	}

	public ImagePlus getSecondImage() {
		return secondImage;
	}

	public void setSecondImage(ImagePlus secondImage) {
		this.secondImage = secondImage;
	}
	
	public void doIt() {
		float result = 0;
		int counter = 0;
		for (int x=0; x<firstImage.getWidth(); x++) {
			for (int y=0; y<firstImage.getHeight(); y++) {
				float value1 = firstImage.getPixel(x,y)[0];
				if (value1==0) continue;
				float value2 = secondImage.getPixel(x,y)[0];
				if (value2==0) continue;
				counter++;
				result = result + Math.abs(value1-value2);
			}
		}
		result = result / (counter * 1.0f);
		difference = this.getDifference();
		difference.reset();
		difference.setHeading(0,"average difference");
		difference.incrementCounter();
		difference.addValue(0, result);
	}
	
	protected void cleanUpInput() {
		super.cleanUpInput();
		this.setFirstImage(null);
		this.setSecondImage(null);
	}
	
	/**
	 * 
	 */
	protected void showResult() {
		IJ.log(this.getDifference().toString());
	}
}
