/*
 * Created on 22.07.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package gui;

import java.util.Observable;
import java.util.Vector;

import operations.Operation;
import applications.Application;

/**
 * @author Volker
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Parameter extends Observable {
	protected Application application;
	protected Operation operation;
	protected int operationIndex;
	protected String[] parameterNames;
	protected Class[] parameterTypes;
	protected Vector possibleInputMaps;
	protected ParameterView view;
	
	
	public Parameter(Operation op, Application app) {
		this.operation = op;
		this.application = app;
		this.initialize();
	}

	/**
	 * 
	 */
	private void initialize() {
		possibleInputMaps = new Vector();
		operationIndex = application.getOperations().indexOf(operation);
		this.parameterNames = operation.getHumanReadableParameterNames();
		this.parameterTypes = operation.getParameterTypes();
		for (int i=0; i<parameterNames.length; i++) {
			this.setupPossibleInputMapFor(i);
		}
	}

	/**
	 * 
	 */
	private void setupPossibleInputMapFor(int parameterIndex) {
		Vector maps = new Vector();
		Class parameterType = parameterTypes[parameterIndex];
		for(int i=0; i<operationIndex; i++) {
			Operation inputOperation = (Operation) application.getOperations().elementAt(i);
			Class[] resultTypes = inputOperation.getResultTypes();
			for (int j=0; j<resultTypes.length; j++) {
				Class resultType = resultTypes[j];
				if (resultType==parameterType) {
					int[] map = new int[2];
					map[0] = i;
					map[1] = j;
					maps.add(map);
				}
			}
		}
		possibleInputMaps.add(maps);
	}
	
	public String[] getParameterNames() {
		return parameterNames;
	}
	
	public String[] getParameterTypes() {
		String[] typeNames = new String[parameterTypes.length]; 
		for (int i=0; i<parameterTypes.length; i++) {
			typeNames[i] = parameterTypes[i].getSimpleName();
		}
		return typeNames;
	}
	
	public Vector getInputOperationsForParameter(int i) {
		Vector result = new Vector();
		Vector maps = (Vector)possibleInputMaps.elementAt(i);
		for (int j=0; j<maps.size(); j++) {
			int[] map = (int[])maps.elementAt(j);
			String name = ((Operation)application.getOperations().elementAt(map[0])).name();
			if ((j>0) && (map[0]==((int[])(maps.elementAt(j-1)))[0])) continue;
			result.add(name);
		}
		return result;
	}
	
	public Vector getResultNamesFor(int parameter, int inputOperation) {
		Vector result = new Vector();
		if (inputOperation<0) return result;
		Vector maps = (Vector)possibleInputMaps.elementAt(parameter);
		int startIndex = this.getStartIndexFor(inputOperation, maps);
		int endIndex = this.getEndIndexFor(inputOperation, maps, startIndex); 
		for(int i=startIndex; i<=endIndex; i++) {
			int[] map = (int[])maps.elementAt(i);	
			Operation inputOp = ((Operation)application.getOperations().elementAt(map[0]));
			String[] resultNames = inputOp.getResultNames();
			result.add(Operation.humanReadableName(resultNames[map[1]]));	
		}
		return result;
	}

	/**
	 * @param inputOperation
	 * @param maps
	 * @return
	 */
	private int getEndIndexFor(int inputOperation, Vector maps, int startIndex) {
		if (startIndex==maps.size()-1) {
			return startIndex;
		}
		for (int i=startIndex+1; i<maps.size(); i++) {
			int[] map = (int[])maps.elementAt(i);
			if ((map[0]!=((int[])(maps.elementAt(i-1)))[0])) return i-1;
		}
		return maps.size()-1;
	}

	/**
	 * @param inputOperation
	 * @param maps
	 */
	private int getStartIndexFor(int inputOperation, Vector maps) {
		if (inputOperation==0) {
			return 0;
		}
		int counter = 0;
		for (int i=1; i<maps.size(); i++) {
			int[] map = (int[])maps.elementAt(i);
			if ((map[0]!=((int[])(maps.elementAt(i-1)))[0])) counter++;
			if (counter==inputOperation) {
				counter = i;
				break;
			}
		}
		return counter;
	}

	/**
	 * 
	 */
	public void show() {
		if (view == null) {
			view = new ParameterView(this);
		}
		view.setVisible(true);
	}
	/**
	 * @return Returns the application.
	 */
	public Application getApplication() {
		return application;
	}
	/**
	 * @return Returns the operation.
	 */
	public Operation getOperation() {
		return operation;
	}
	
	public String getMappingTextForParameters() {
		String result = "";
		for (int i=0; i<this.parameterNames.length; i++) {
			result = result + this.getMappingTextForParameter(i);
			if (i<parameterNames.length-1) {
				result = result + "\n";
			}
		}
		return result;
	}
	public String getMappingTextForParameter(int i) {
		String parameterName = parameterNames[i];
		int[] map = (int[]) this.operation.getInputMap().elementAt(i);
		String result = "(" + parameterTypes[i].getSimpleName() + ") " + parameterNames[i] + " := ";
		if (map[0]==-1 || map[1]==-1) {
			result = result + " null";
		} else {
			Operation inputOperation = (Operation) application.getOperations().elementAt(map[0]);
			String resultName = inputOperation.getResultNames()[map[1]];
			result = result + inputOperation.name() + " : " + resultName;
		}
		return result;
	}

	/**
	 * @param parameterIndex
	 * @param operationIndex2
	 * @param resultIndex
	 */
	public void mapOperationInput(int parameterIndex, int sourceOperationIndex, int resultIndex) {
		Vector map = (Vector)possibleInputMaps.elementAt(parameterIndex);
		int[] mapping = findMappingFor(map, sourceOperationIndex, resultIndex);
		this.operation.setInputForParameterTo(parameterIndex, mapping[0], mapping[1]);
	}

	/**
	 * @param map
	 * @param sourceOperationIndex
	 * @param resultIndex
	 * @return
	 */
	private int[] findMappingFor(Vector map, int sourceOperationIndex, int resultIndex) {
		int startIndex = this.getStartIndexFor(sourceOperationIndex, map);
		return ((int[])map.elementAt(startIndex+resultIndex));
	}

	
}
