// **********************************************************************
// COPYRIGHT NOTICE AND DISCLAIMER
//
// Copyright (c) Erik Meijering. All rights reserved.
//
// Permission to copy and use this software and accompanying
// documentation for educational, research, and not-for-profit
// purposes, without fee and without a signed licensing agreement,
// is hereby granted, provided that the above copyright notice, this
// paragraph and the following two paragraphs appear in all copies.
// The copyright holder is free to make upgraded or improved versions
// of the software available for a fee or commercially only.
//
// IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
// AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
// THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// **********************************************************************

package imagescience.features;

import imagescience.utilities.Messenger;

/** Contains some basic fields and methods used by the other classes in
    the package. */
public final class Features {

    private static final double VERSION = 1.2;

    private static boolean bShowDebug = false;
    private static boolean bSepSmooth = false;

    static final Messenger msg = new Messenger();

    /** Prints the current version of <code>Features</code>. This will work only if
        the user has chosen to let <code>Features</code> generate debug messages. */
    public static void printVersion() {	msg.debug("\nFeatures "+VERSION); }

    /** Prints the current version of <code>Features</code>, appended with the
        argument. This will work only if the user has chosen to let
        <code>Features</code> generate debug messages. */
    public static void printVersion(final String append) { msg.debug("\nFeatures "+VERSION+": "+append); }

    /** Returns the current version of <code>Features</code>. */
    public static double version() { return VERSION; }

    /** Method to specify whether or not debug messages should be generated. */
    public static void showDebug(final boolean show) { bShowDebug = show; msg.debug(bShowDebug); }

    /** Indicates whether or not debug messages are generated. */
    public static boolean showDebug() {	return bShowDebug; }

    /** Method to specify whether or not to seperate smoothing and differentiation. */
    public static void sepSmooth(final boolean sep) { bSepSmooth = sep; }

    /** Indicates whether or not smoothing and differentiation are seperated. */
    public static boolean sepSmooth() {	return bSepSmooth; }

}
