//**********************************************************************
//COPYRIGHT NOTICE AND DISCLAIMER
//
//Copyright (c) Erik Meijering. All rights reserved.
//
//Permission to copy and use this software and accompanying
//documentation for educational, research, and not-for-profit
//purposes, without fee and without a signed licensing agreement,
//is hereby granted, provided that the above copyright notice, this
//paragraph and the following two paragraphs appear in all copies.
//The copyright holder is free to make upgraded or improved versions
//of the software available for a fee or commercially only.
//
//IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
//DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
//AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
//POSSIBILITY OF SUCH DAMAGE.
//
//THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
//INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
//SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
//THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
//SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//**********************************************************************
package neuronJgui;

import ij.gui.ImageCanvas;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;

public final class Tracing {

    private int iCapacity = 20;
    private final int iCapInc = 20;
    private int iSize = 0;
    private Segment[] sarray = null;

    private boolean hili = false;
    private boolean select = false;
    private boolean changed = false;

    private int type = 0;
    private int cluster = 0;
    private String label = "Default";

    private static int lastID = 0;
    private int ID;

    Tracing() {
	ID = ++lastID;
	sarray = new Segment[iCapacity];
    }

    Tracing(final int capacity) {
	ID = ++lastID;
	iCapacity = capacity;
	sarray = new Segment[iCapacity];
    }

    void id(final int id) { ID = id; if (ID > lastID) lastID = ID; NJ.save = true; }

    int id() { return ID; }

    static void resetID() { lastID = 0; }

    void add(final Segment segment) {
	if (iSize == iCapacity) inccap();
	sarray[iSize++] = segment;
	changed = true;
	NJ.save = true;
    }
	
    private void inccap() {
	iCapacity += iCapInc;
	final Segment[] newarray = new Segment[iCapacity];
	for (int i=0; i<iSize; ++i) newarray[i] = sarray[i];
	sarray = newarray;
    }
	
    Segment element(final int index) { return sarray[index]; }
	
    int nrsegments() { return iSize; }

    double length() {
	double length = 0.0;
	for (int s=0; s<iSize; ++s)
	    length += sarray[s].length();
	return length;
    }

    boolean nearby(final Point point) {
	for (int s=0; s<iSize; ++s)
	    if (sarray[s].nearby(point)) return true;
	return false;
    }

    boolean changed() {	return changed; }

    void select(final boolean select) {
	if (this.select != select) {
	    this.select = select;
	    changed = true;
	}
    }

    boolean selected() { return select; }

    void highlight(final boolean hili) {
	if (this.hili != hili) {
	    this.hili = hili;
	    changed = true;
	}
    }

    boolean highlighted() { return hili; }

    void type(final int type) {
	if (this.type != type) {
	    this.type = type;
	    changed = true;
	    NJ.save = true;
	}
    }

    int type() { return type; }
	
    void cluster(final int cluster) {
	if (this.cluster != cluster) {
	    this.cluster = cluster;
	    NJ.save = true;
	}
    }

    int cluster() { return cluster; }

    void label(final String label) {
	if (!this.label.equals(label)) {
	    this.label = label;
	    NJ.save = true;
	}
    }

    String label() { return label; }

    void draw(final Graphics g, final ImageCanvas imc) {
	final Color drawcolor = (hili || select) ? NJ.HIGHLIGHTCOLOR : NJ.typecolors[type];
	for (int s=0; s<iSize; ++s) sarray[s].draw(g,imc,drawcolor);
	changed = false;
    }

}
