//**********************************************************************
//COPYRIGHT NOTICE AND DISCLAIMER
//
//Copyright (c) Erik Meijering. All rights reserved.
//
//Permission to copy and use this software and accompanying
//documentation for educational, research, and not-for-profit
//purposes, without fee and without a signed licensing agreement,
//is hereby granted, provided that the above copyright notice, this
//paragraph and the following two paragraphs appear in all copies.
//The copyright holder is free to make upgraded or improved versions
//of the software available for a fee or commercially only.
//
//IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
//DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
//AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
//POSSIBILITY OF SUCH DAMAGE.
//
//THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
//INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
//SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
//THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
//SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//**********************************************************************
package neuronJgui;

import ij.IJ;
import ij.gui.GUI;
import ij.gui.MultiLineLabel;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

final public class YesNoDialog extends Dialog implements ActionListener, KeyListener {

    private Button yesButton;
    private Button noButton;

    private boolean yesPressed = false;
    private boolean noPressed = false;

    // Builds the dialog for specifying the parameters for derivative computing.
    public YesNoDialog(final String title, final String question) {

	super(IJ.getInstance(),"NeuronJ: "+title,true);
        setLayout(new BorderLayout());

	// Add question:
        final Panel questPanel = new Panel();
        questPanel.setLayout(new FlowLayout(FlowLayout.LEFT,10,10));
        final MultiLineLabel questLabel = new MultiLineLabel(question);
        questLabel.setFont(new Font("Dialog",Font.PLAIN,12));
        questPanel.add(questLabel);
        add(questPanel,BorderLayout.NORTH);
        
	// Add No and Yes buttons:
        final Panel yesnoPanel = new Panel();
        yesnoPanel.setLayout(new FlowLayout(FlowLayout.CENTER,15,10));
        yesButton = new Button("  Yes  ");
        yesButton.addActionListener(this);
        yesButton.addKeyListener(this);
        yesnoPanel.add(yesButton);
        noButton = new Button("   No   ");
        noButton.addActionListener(this);
        noButton.addKeyListener(this);
        yesnoPanel.add(noButton);
        add(yesnoPanel,BorderLayout.SOUTH);

	// Pack and show:
	pack();
	GUI.center(this);
	setResizable(false);
	show();
    }
    
    public void actionPerformed(final ActionEvent e) {

	if (e.getSource() == yesButton) yesPressed = true;
	else if (e.getSource() == noButton) noPressed = true;

	setVisible(false);
	dispose();
    }
    
    public void keyTyped(KeyEvent e) {}

    public void keyPressed(KeyEvent e) {

	if (e.getKeyCode() == KeyEvent.VK_Y) yesPressed = true;
	else if (e.getKeyCode() == KeyEvent.VK_N) noPressed = true;
	else if (e.getKeyCode() == KeyEvent.VK_ENTER)
	    if (yesButton.hasFocus()) yesPressed = true;
	    else if (noButton.hasFocus()) noPressed = true;
	
	if (yesPressed || noPressed) {
	    setVisible(false);
	    dispose();
	}
    }

    public void keyReleased(KeyEvent e) {}

    public boolean yesPressed() { return yesPressed; }
    
    boolean noPressed() { return noPressed; }

}
